import random
from _ast import mod

import numpy as np
from random import gauss
from random import seed
from random import randint, uniform
import matplotlib.pylab as plt
import sys

def generateRwSeries(lengthRW):

    RW = list()
    for i in range(lengthRW):
        if(i==0):
            RW.append(round(np.random.normal(), 2))
        else:
            val = RW[i-1] + round(np.random.normal(), 2)
            RW.append(val)

    mu = np.mean(RW)
    stdev = np.std(RW)
    RWnorm = (RW - mu) / stdev

    x= [i for i in range(len(RWnorm))]
    plt.plot(x, RWnorm )
    plt.xlabel('domain')
    plt.ylabel('value')
    plt.axis('tight')
    plt.show()

    return RWnorm


def generateAnomaly(lengthAnomaly,percentageNoise):


    randShift = uniform(0, 4*np.pi)
    x = np.linspace( -np.pi-randShift, np.pi+randShift, lengthAnomaly)
    anomaly = np.sin(x);

    noise = (percentageNoise / 100) * np.asarray(random.sample(list(anomaly), lengthAnomaly))
    anomaly = anomaly + noise

    mu = np.mean(anomaly)
    stdev = np.std(anomaly)
    sinAnomNorm = (anomaly - mu) / stdev




    """
    plt.plot(x, sinAnomNorm)
    plt.title("Anomaly length "+str(lengthAnomaly))
    plt.xlabel('Angle [rad]')
    plt.ylabel('sin(x)')
    plt.axis('tight')
    plt.show()
    """
    return sinAnomNorm;


def generateSinPeriod(lengthPeriod,percentageNoise):
    x = np.linspace(-np.pi, np.pi, lengthPeriod)
    sinPeriod = np.sin(x);


    noise = (percentageNoise/100) * np.asarray(random.sample(list(sinPeriod), lengthPeriod))
    sinPeriod = sinPeriod + noise

    mu = np.mean(sinPeriod)
    stdev = np.std(sinPeriod)
    sinPeriodNorm = (sinPeriod - mu) / stdev

    """"
    plt.plot(x, sinPeriodNorm)
    plt.xlabel('Angle [rad]')
    plt.ylabel('sin(x)')
    plt.axis('tight')
    plt.show()
    """

    return sinPeriodNorm;


print(str(__name__))
if __name__ == "__main__":

    if len(sys.argv) == 7:
        print("start!")
        lengthCorrectPart = int(sys.argv[1])
        lengthPatternSin = int(sys.argv[2])
        numberAnomaly =  int(sys.argv[3])
        lengthAnomaly = int(sys.argv[4])
        percNoise = int(sys.argv[5])
        nameFileDataset = sys.argv[6]

        lengthDataset = (lengthCorrectPart+(lengthAnomaly*numberAnomaly))

        RwSeries = generateRwSeries(lengthDataset)
        patternSin = generateSinPeriod(lengthPatternSin,percNoise)
        numberPeriods = int(lengthCorrectPart/lengthPatternSin)
        sinSignal = np.tile(patternSin,numberPeriods)
        #sinSignal  = sinSignal + RwSeries


        posAnomaly = random.sample(range(1,lengthCorrectPart ), numberAnomaly)
        posAnomaly.sort()

        for i in range(len(posAnomaly)):
            a = generateAnomaly(lengthAnomaly,percNoise)
            posA = (posAnomaly[i] - 1)
            if i>=1:
                # shift offset since adding anomly augments the number of points in the correct signal
                posA = posA + lengthAnomaly*i
                posAnomaly[i] = posA
            sinSignal = np.concatenate((np.array(sinSignal[0:posA]), np.array(a), np.array(sinSignal[posA:len(sinSignal)])))

        sinSignal = sinSignal.tolist()

        sum = 0
        for i in range(len(RwSeries)):
            ind = i + 1
            sum = sum + RwSeries[i]
            if ind % lengthPatternSin == 0:
                avg = sum / lengthPatternSin
                ind2 = (ind - lengthPatternSin)
                for j in sinSignal[(ind - lengthPatternSin):ind]:
                    sinSignal[ind2] = sinSignal[ind2] + avg
                    ind2 += 1
                sum = 0

        nameFileDataset = nameFileDataset + "_Length_"+str(len(sinSignal))+"_AnomalyL_"+str(lengthAnomaly)+"_AnomalyN_"+str(len(posAnomaly))+"_NoisePerc_"+str(percNoise)
        nameFileAnnotations = nameFileDataset  +"_Annotations.txt"
        nameFileDataset = nameFileDataset + ".ts"


        fileDAtaset = open(nameFileDataset,"w")
        bInit = True
        for value in sinSignal:
            if bInit:
                fileDAtaset.write(str(value))
                bInit = False
            else:
                fileDAtaset.write("\n"+str(value))

        fileDAtaset.close()
        fileAnn = open(nameFileAnnotations,"w")
        bInit = True
        for posAn in posAnomaly:
            if bInit:
                fileAnn.write(str(posAn))
                bInit = False
            else:
                fileAnn.write("\n" + str(posAn))


        fileAnn.close()

        x = [i for i in range(len(sinSignal))]
        plt.plot(x, sinSignal)
        plt.xlabel('domain')
        plt.ylabel('value')
        plt.axis('tight')
        print("length dataset: "+str(len(sinSignal)))
        for posAn in posAnomaly:
            xanom = [i for i in range(posAn,posAn+lengthAnomaly)]
            yAnom = sinSignal[posAn:(posAn+lengthAnomaly)]
            print("pos anomaly: "+str(posAn))
            plt.plot(xanom, yAnom, color="red")
        plt.show()

    else:
        print("arguments in order must be: [length of series to generate w/o anomaly], [length of cycle], "
              "[number of anomaly to generate], [length of anomaly], [% of noise to add to correct data], [datasetName w/o extension].")
